@@ -9,6 +9,7 @@ class EveCategoryId(IntEnum):
 
 class EveGroupId(IntEnum):
     MOON = 8
+    MINERAL = 18
     REFINERY = 1406
     UBIQUITOUS_MOON_ASTEROIDS = 1884
     COMMON_MOON_ASTEROIDS = 1920
@@ -18,6 +19,19 @@ class EveGroupId(IntEnum):
 
 
 class EveTypeId(IntEnum):
+    ATHANOR = 35835
+    CHROMITE = 45501
+    EUXENITE = 45495
+    XENOTIME = 45510
+    BITUMENS = 45492
+    CINNABAR = 45506
+    CUBIC_BISTOT = 46676
+    FLAWLESS_ARKONOR = 46678
+    STABLE_VELDSPAR = 46689
+    ZEOLITES = 45490
+    BRIMFUL_ZEOLITES = 46280
+    GLISTENING_ZEOLITES = 46281
+
     MOON = 14
 
 
